@@ -1,6 +1,8 @@
 #from ctx_base import StandardBaseContext
 
-from libmp import (MPZ, MPZ_ZERO, MPZ_ONE, int_types, repr_dps,
+from .libmp.backend import basestring, exec_
+
+from .libmp import (MPZ, MPZ_ZERO, MPZ_ONE, int_types, repr_dps,
     round_floor, round_ceiling, dps_to_prec, round_nearest, prec_to_dps,
     ComplexResult, to_pickable, from_pickable, normalize,
     from_int, from_float, from_str, to_int, to_float, to_str,
@@ -23,8 +25,8 @@ from libmp import (MPZ, MPZ_ZERO, MPZ_ONE, int_types, repr_dps,
     mpf_glaisher, mpf_twinprime, mpf_mertens,
     int_types)
 
-import rational
-import function_docs
+from . import rational
+from . import function_docs
 
 new = object.__new__
 
@@ -138,10 +140,12 @@ class _mpf(mpnumeric):
     def __hash__(s): return mpf_hash(s._mpf_)
     def __int__(s): return int(to_int(s._mpf_))
     def __long__(s): return long(to_int(s._mpf_))
-    def __float__(s): return to_float(s._mpf_)
+    def __float__(s): return to_float(s._mpf_, rnd=s.context._prec_rounding[1])
     def __complex__(s): return complex(float(s))
     def __nonzero__(s): return s._mpf_ != fzero
 
+    __bool__ = __nonzero__
+
     def __abs__(s):
         cls, new, (prec, rounding) = s._ctxdata
         v = new(cls)
@@ -224,6 +228,8 @@ class _mpf(mpnumeric):
     def to_fixed(self, prec):
         return to_fixed(self._mpf_, prec)
 
+    def __round__(self, *args):
+        return round(float(self), *args)
 
 mpf_binary_op = """
 def %NAME%(self, other):
@@ -275,7 +281,7 @@ def binary_op(name, with_mpf='', with_int='', with_mpc=''):
     code = code.replace("%WITH_MPF%", with_mpf)
     code = code.replace("%NAME%", name)
     np = {}
-    exec code in globals(), np
+    exec_(code, globals(), np)
     return np[name]
 
 _mpf.__eq__ = binary_op('__eq__',
@@ -345,7 +351,7 @@ class _constant(_mpf):
         return self.func(prec, rounding)
 
     def __repr__(self):
-        return "<%s: %s~>" % (self.name, self.context.nstr(self))
+        return "<%s: %s~>" % (self.name, self.context.nstr(self(dps=15)))
 
 
 class _mpc(mpnumeric):
@@ -389,7 +395,7 @@ class _mpc(mpnumeric):
         return "(%s)" % mpc_to_str(s._mpc_, s.context._str_digits)
 
     def __complex__(s):
-        return mpc_to_complex(s._mpc_)
+        return mpc_to_complex(s._mpc_, rnd=s.context._prec_rounding[1])
 
     def __pos__(s):
         cls, new, (prec, rounding) = s._ctxdata
@@ -418,6 +424,8 @@ class _mpc(mpnumeric):
     def __nonzero__(s):
         return mpc_is_nonzero(s._mpc_)
 
+    __bool__ = __nonzero__
+
     def __hash__(s):
         return mpc_hash(s._mpc_)
 
@@ -570,7 +578,7 @@ class _mpc(mpnumeric):
 complex_types = (complex, _mpc)
 
 
-class PythonMPContext:
+class PythonMPContext(object):
 
     def __init__(ctx):
         ctx._prec_rounding = [53, round_nearest]
@@ -894,7 +902,7 @@ class PythonMPContext:
             >>> B = [1, -1, 2]
             >>> fdot(A, B)
             mpf('6.5')
-            >>> zip(A, B)
+            >>> list(zip(A, B))
             [(2, 1), (1.5, -1), (3, 2)]
             >>> fdot(_)
             mpf('6.5')
@@ -1119,3 +1127,16 @@ class PythonMPContext:
             else:
                 raise TypeError("requires an mpf/mpc")
 
+
+# Register with "numbers" ABC
+#     We do not subclass, hence we do not use the @abstractmethod checks. While
+#     this is less invasive it may turn out that we do not actually support
+#     parts of the expected interfaces.  See
+#     http://docs.python.org/2/library/numbers.html for list of abstract
+#     methods.
+try:
+    import numbers
+    numbers.Complex.register(_mpc)
+    numbers.Real.register(_mpf)
+except ImportError:
+    pass
