@@ -33,7 +33,7 @@ using the following relations:
 
 .. math ::
 
-    \tau = -i \frac{K(1-m)}{K(m)}
+    \tau = i \frac{K(1-m)}{K(m)}
 
 .. math ::
 
@@ -62,7 +62,7 @@ between the various parameters (:func:`~mpmath.qfrom`, :func:`~mpmath.mfrom`,
 
 """
 
-from functions import defun, defun_wrapped
+from .functions import defun, defun_wrapped
 
 def nome(ctx, m):
     m = ctx.convert(m)
@@ -687,7 +687,7 @@ def elliprc(ctx, x, y, pv=True):
 
     .. math ::
 
-        R_C(x,y) = 
+        R_C(x,y) =
         \begin{cases}
           \dfrac{1}{\sqrt{y-x}}
             \cos^{-1}\left(\sqrt{\dfrac{x}{y}}\right),   & x < y \\
@@ -896,13 +896,20 @@ def elliprg(ctx, x, y, z):
     x = ctx.convert(x)
     y = ctx.convert(y)
     z = ctx.convert(z)
-    if not z: x, z = z, x
-    if not z: y, z = x, y
-    if not z: return ctx.inf
+    zeros = (not x) + (not y) + (not z)
+    if zeros == 3:
+        return (x+y+z)*0
+    if zeros == 2:
+        if x: return 0.5*ctx.sqrt(x)
+        if y: return 0.5*ctx.sqrt(y)
+        return 0.5*ctx.sqrt(z)
+    if zeros == 1:
+        if not z:
+            x, z = z, x
     def terms():
         T1 = 0.5*z*ctx.elliprf(x,y,z)
         T2 = -0.5*(x-z)*(y-z)*ctx.elliprd(x,y,z)/3
-        T3 = 0.5*ctx.sqrt(x*y/z)
+        T3 = 0.5*ctx.sqrt(x)*ctx.sqrt(y)/ctx.sqrt(z)
         return T1,T2,T3
     return ctx.sum_accurately(terms)
 
@@ -920,7 +927,7 @@ def ellipf(ctx, phi, m):
 
     .. math ::
 
-        F(\phi,m) = \int_0^{\sin z}
+        F(\phi,m) = \int_0^{\sin \phi}
         \frac{dt}{\left(\sqrt{1-t^2}\right)\left(\sqrt{1-mt^2}\right)}.
 
     The function reduces to a complete elliptic integral of the first kind
@@ -932,7 +939,7 @@ def ellipf(ctx, phi, m):
 
     In the defining integral, it is assumed that the principal branch
     of the square root is taken and that the path of integration avoids
-    crossing any branch cuts. Outside `-\pi/2 \le \Re(z) \le \pi/2`,
+    crossing any branch cuts. Outside `-\pi/2 \le \Re(\phi) \le \pi/2`,
     the function extends quasi-periodically as
 
     .. math ::
@@ -1211,7 +1218,7 @@ def ellippi(ctx, *args):
 
     In the defining integral, it is assumed that the principal branch
     of the square root is taken and that the path of integration avoids
-    crossing any branch cuts. Outside `-\pi/2 \le \Re(z) \le \pi/2`,
+    crossing any branch cuts. Outside `-\pi/2 \le \Re(\phi) \le \pi/2`,
     the function extends quasi-periodically as
 
     .. math ::
@@ -1282,6 +1289,19 @@ def ellippi(ctx, *args):
         >>> ellippi(0.5, 5+6j-2*pi, -7-8j)
         (-0.3612856620076747660410167 + 0.5217735339984807829755815j)
 
+    Some degenerate cases::
+
+        >>> ellippi(1,1)
+        +inf
+        >>> ellippi(1,0)
+        +inf
+        >>> ellippi(1,2,0)
+        +inf
+        >>> ellippi(1,2,1)
+        +inf
+        >>> ellippi(1,0,1)
+        0.0
+
     """
     if len(args) == 2:
         n, m = args
@@ -1295,7 +1315,10 @@ def ellippi(ctx, *args):
         if ctx.isnan(n) or ctx.isnan(z) or ctx.isnan(m):
             raise ValueError
         if complete:
-            if m == 0: return ctx.pi/(2*ctx.sqrt(1-n))
+            if m == 0:
+                if n == 1:
+                    return ctx.inf
+                return ctx.pi/(2*ctx.sqrt(1-n))
             if n == 0: return ctx.ellipk(m)
             if ctx.isinf(n) or ctx.isinf(m): return ctx.zero
         else:
@@ -1305,7 +1328,10 @@ def ellippi(ctx, *args):
         if ctx.isinf(n) or ctx.isinf(z) or ctx.isinf(m):
             raise ValueError
     if complete:
-        if m == 1: return -ctx.inf/ctx.sign(n-1)
+        if m == 1:
+            if n == 1:
+                return ctx.inf
+            return -ctx.inf/ctx.sign(n-1)
         away = False
     else:
         x = z.real
@@ -1316,6 +1342,8 @@ def ellippi(ctx, *args):
         d = ctx.nint(x/pi)
         z = z-pi*d
         P = 2*d*ctx.ellippi(n,m)
+        if ctx.isinf(P):
+            return ctx.inf
     else:
         P = 0
     def terms():
