@@ -1,6 +1,6 @@
-# TODO: interpret list as vectors (for multiplication)
+from ..libmp.backend import xrange
 
-import itertools
+# TODO: interpret list as vectors (for multiplication)
 
 rowsep = '\n'
 colsep = '  '
@@ -70,12 +70,12 @@ class _matrix(object):
         >>> A
         matrix(
         [['0.0', '0.0'],
-         ['0.0', '(1.0 + 1.0j)']])
+         ['0.0', mpc(real='1.0', imag='1.0')]])
 
     You can use the keyword ``force_type`` to change the function which is
     called on every new element:
 
-        >>> matrix(2, 5, force_type=int)
+        >>> matrix(2, 5, force_type=int) # doctest: +SKIP
         matrix(
         [[0, 0, 0, 0, 0],
          [0, 0, 0, 0, 0]])
@@ -92,8 +92,8 @@ class _matrix(object):
 
         >>> matrix([[1, 2.5], [1j, mpf(2)]], force_type=None)
         matrix(
-        [[1, 2.5],
-         [1j, '2.0']])
+        [['1.0', '2.5'],
+         [mpc(real='0.0', imag='1.0'), '2.0']])
 
     Convenient advanced functions are available for creating various standard
     matrices, see ``zeros``, ``ones``, ``diag``, ``eye``, ``randmatrix`` and
@@ -290,6 +290,8 @@ class _matrix(object):
         # determinant
         self._LU = None
         convert = kwargs.get('force_type', self.ctx.convert)
+        if not convert:
+            convert = lambda x: x
         if isinstance(args[0], (list, tuple)):
             if isinstance(args[0][0], (list, tuple)):
                 # interpret nested list as matrix
@@ -311,7 +313,8 @@ class _matrix(object):
             if len(args) == 1:
                 self.__rows = self.__cols = args[0]
             else:
-                assert isinstance(args[1], int), 'expected int'
+                if not isinstance(args[1], int):
+                    raise TypeError("expected int")
                 self.__rows = args[0]
                 self.__cols = args[1]
         elif isinstance(args[0], _matrix):
@@ -319,7 +322,6 @@ class _matrix(object):
             self.__data = A._matrix__data
             self.__rows = A._matrix__rows
             self.__cols = A._matrix__cols
-            convert = kwargs.get('force_type', self.ctx.convert)
             for i in xrange(A.__rows):
                 for j in xrange(A.__cols):
                     A[i,j] = convert(A[i,j])
@@ -404,7 +406,7 @@ class _matrix(object):
 
     def __get_element(self, key):
         '''
-        Fast extraction of the i,j element from the matrix   
+        Fast extraction of the i,j element from the matrix
             This function is for private use only because is unsafe:
                 1. Does not check on the value of key it expects key to be a integer tuple (i,j)
                 2. Does not check bounds
@@ -429,11 +431,11 @@ class _matrix(object):
 
 
     def __getitem__(self, key):
-        ''' 
+        '''
             Getitem function for mp matrix class with slice index enabled
             it allows the following assingments
             scalar to a slice of the matrix
-         B = A[:,2:6] 
+         B = A[:,2:6]
         '''
         # Convert vector to matrix indexing
         if isinstance(key, int) or isinstance(key,slice):
@@ -444,14 +446,14 @@ class _matrix(object):
                 key = (key, 0)
             else:
                 raise IndexError('insufficient indices for matrix')
-            
+
         if isinstance(key[0],slice) or isinstance(key[1],slice):
 
             #Rows
             if isinstance(key[0],slice):
                 #Check bounds
-                if (key[0].start >= 0 or key[0].start is None) and \
-                    (key[0].stop <= self.__rows+1 or key[0].stop is None):
+                if (key[0].start is None or key[0].start >= 0) and \
+                    (key[0].stop is None or key[0].stop <= self.__rows+1):
                     # Generate indices
                     rows = xrange(*key[0].indices(self.__rows))
                 else:
@@ -459,31 +461,31 @@ class _matrix(object):
             else:
                 # Single row
                 rows = [key[0]]
-            
+
             # Columns
             if isinstance(key[1],slice):
                 # Check bounds
-                if (key[1].start >= 0 or key[1].start is None) and \
-                    (key[1].stop <= self.__cols+1 or key[1].stop is None):
+                if (key[1].start is None or key[1].start >= 0) and \
+                    (key[1].stop is None or key[1].stop <= self.__cols+1):
                     # Generate indices
                     columns = xrange(*key[1].indices(self.__cols))
                 else:
                     raise IndexError('Column index out of bounds')
-                
+
             else:
                 # Single column
                 columns = [key[1]]
 
             # Create matrix slice
-            m = self.ctx.matrix(len(rows),len(columns)) 
+            m = self.ctx.matrix(len(rows),len(columns))
 
             # Assign elements to the output matrix
             for i,x in enumerate(rows):
                 for j,y in enumerate(columns):
                     m.__set_element((i,j),self.__get_element((x,y)))
-            
+
             return m
-            
+
         else:
             # single element extraction
             if key[0] >= self.__rows or key[1] >= self.__cols:
@@ -500,7 +502,6 @@ class _matrix(object):
         # A[:,2:6] = 2.5
         #  submatrix to matrix (the value matrix should be the same size as the slice size)
         # A[3,:] = B   where A is n x m  and B is n x 1
-        
         # Convert vector to matrix indexing
         if isinstance(key, int) or isinstance(key,slice):
             # only sufficent for vectors
@@ -510,15 +511,13 @@ class _matrix(object):
                 key = (key, 0)
             else:
                 raise IndexError('insufficient indices for matrix')
-                
-        # Slice indexing            
+        # Slice indexing
         if isinstance(key[0],slice) or isinstance(key[1],slice):
-            
             # Rows
             if isinstance(key[0],slice):
                 # Check bounds
-                if (key[0].start >= 0 or key[0].start is None) and \
-                    (key[0].stop <= self.__rows+1 or key[0].stop is None):
+                if (key[0].start is None or key[0].start >= 0) and \
+                    (key[0].stop is None or key[0].stop <= self.__rows+1):
                     # generate row indices
                     rows = xrange(*key[0].indices(self.__rows))
                 else:
@@ -526,37 +525,33 @@ class _matrix(object):
             else:
                 # Single row
                 rows = [key[0]]
-            
             # Columns
             if isinstance(key[1],slice):
                 # Check bounds
-                if (key[1].start >= 0 or key[1].start is None) and \
-                    (key[1].stop <= self.__cols+1 or key[1].stop is None):
+                if (key[1].start is None or key[1].start >= 0) and \
+                    (key[1].stop is None or key[1].stop <= self.__cols+1):
                     # Generate column indices
                     columns = xrange(*key[1].indices(self.__cols))
                 else:
                     raise IndexError('Column index out of bounds')
-                
             else:
                 # Single column
                 columns = [key[1]]
-
-                                 
             # Assign slice with a scalar
             if isinstance(value,self.ctx.matrix):
                 # Assign elements to matrix if input and output dimensions match
                 if len(rows) == value.rows and len(columns) == value.cols:
                     for i,x in enumerate(rows):
                         for j,y in enumerate(columns):
-                            self.__set_element( (x,y),value.__get_element((i,j)))
+                            self.__set_element((x,y), value.__get_element((i,j)))
                 else:
                     raise ValueError('Dimensions do not match')
             else:
                 # Assign slice with scalars
                 value = self.ctx.convert(value)
-                for index in itertools.product(rows,columns):
-                    self.__set_element( index ,value)
-        
+                for i in rows:
+                    for j in columns:
+                        self.__set_element((i,j), value)
         else:
             # Single element assingment
             # Check bounds
@@ -568,11 +563,10 @@ class _matrix(object):
                 self.__data[key] = value
             elif key in self.__data:
                 del self.__data[key]
-        
+
         if self._LU:
             self._LU = None
         return
-        
 
     def __iter__(self):
         for i in xrange(self.__rows):
@@ -600,7 +594,8 @@ class _matrix(object):
 
     def __rmul__(self, other):
         # assume other is scalar and thus commutative
-        assert not isinstance(other, self.ctx.matrix)
+        if isinstance(other, self.ctx.matrix):
+            raise TypeError("other should not be type of ctx.matrix")
         return self.__mul__(other)
 
     def __pow__(self, other):
@@ -689,7 +684,7 @@ class _matrix(object):
         return self.__rows
 
     def __setrows(self, value):
-        for key in self.__data.copy().iterkeys():
+        for key in self.__data.copy():
             if key[0] >= value:
                 del self.__data[key]
         self.__rows = value
@@ -700,7 +695,7 @@ class _matrix(object):
         return self.__cols
 
     def __setcols(self, value):
-        for key in self.__data.copy().iterkeys():
+        for key in self.__data.copy():
             if key[1] >= value:
                 del self.__data[key]
         self.__cols = value
@@ -880,8 +875,10 @@ class MatrixMethods(object):
         """
         Extend matrix A with column b and return result.
         """
-        assert isinstance(A, ctx.matrix)
-        assert A.rows == len(b)
+        if not isinstance(A, ctx.matrix):
+            raise TypeError("A should be a type of ctx.matrix")
+        if A.rows != len(b):
+            raise ValueError("Value should be equal to len(b)")
         A = A.copy()
         A.cols += 1
         for i in xrange(A.rows):
