@@ -70,12 +70,12 @@ def testit(importdir='', testdir=''):
         sys.path.insert(1, testdir)
     import os.path
     import mpmath
-    print "mpmath imported from", os.path.dirname(mpmath.__file__)
-    print "mpmath backend:", mpmath.libmp.backend.BACKEND
-    print "mpmath mp class:", repr(mpmath.mp)
-    print "mpmath version:", mpmath.__version__
-    print "Python version:", sys.version
-    print
+    print("mpmath imported from %s" % os.path.dirname(mpmath.__file__))
+    print("mpmath backend: %s" % mpmath.libmp.backend.BACKEND)
+    print("mpmath mp class: %s" % repr(mpmath.mp))
+    print("mpmath version: %s" % mpmath.__version__)
+    print("Python version: %s" % sys.version)
+    print("")
     if "-py" in sys.argv:
         sys.argv.remove('-py')
         import py
@@ -115,12 +115,12 @@ def testit(importdir='', testdir=''):
         modules.sort()
         tstart = clock()
         for priority, name, module in modules:
-            print name
+            print(name)
             for f in sorted(module.__dict__.keys()):
                 if f.startswith('test_'):
                     if coverage and ('numpy' in f):
                         continue
-                    print "   ", f[5:].ljust(25),
+                    sys.stdout.write("    " + f[5:].ljust(25) + " ")
                     t1 = clock()
                     try:
                         module.__dict__[f]()
@@ -128,15 +128,15 @@ def testit(importdir='', testdir=''):
                         etype, evalue, trb = sys.exc_info()
                         if etype in (KeyboardInterrupt, SystemExit):
                             raise
-                        print
-                        print "TEST FAILED!"
-                        print
+                        print("")
+                        print("TEST FAILED!")
+                        print("")
                         traceback.print_exc()
                     t2 = clock()
-                    print "ok", "      ", ("%.7f" % (t2-t1)), "s"
+                    print("ok " + "       " + ("%.7f" % (t2-t1)) + " s")
         tend = clock()
-        print
-        print "finished tests in", ("%.2f" % (tend-tstart)), "seconds"
+        print("")
+        print("finished tests in " + ("%.2f" % (tend-tstart)) + " seconds")
         # clean sys.path
         if importdir:
             sys.path.remove(importdir)
@@ -156,4 +156,3 @@ if __name__ == '__main__':
         r.write_results(show_missing=True, summary=True, coverdir="/tmp")
     else:
         testit(importdir, testdir)
-
