@@ -12,6 +12,7 @@ def test_bessel():
     assert j1(1000).ae(0.00472831190708952392)
     assert j1(-25).ae(0.125350249580289905)
     assert besselj(5,1).ae(0.000249757730211234431)
+    assert besselj(5+0j,1).ae(0.000249757730211234431)
     assert besselj(5,pi).ae(0.0521411843671184747)
     assert besselj(5,1000).ae(0.00502540694523318607)
     assert besselj(5,-25).ae(0.0660079953984229934)
@@ -70,6 +71,25 @@ def test_bessel():
     assert besselk(0,3+4j).ae(-0.007239051213570155013+0.026510418350267677215j)
     assert besselk(0,j).ae(-0.13863371520405399968-1.20196971531720649914j)
     assert (besselk(3, 10**10) * mpf(10)**4342944824).ae(1.1628981033356187851)
+    # test for issue 331, bug reported by Michael Hartmann
+    for n in range(10,100,10):
+        mp.dps = n
+        assert besseli(91.5,24.7708).ae("4.00830632138673963619656140653537080438462342928377020695738635559218797348548092636896796324190271316137982810144874264e-41")
+
+def test_bessel_zeros():
+    mp.dps = 15
+    assert besseljzero(0,1).ae(2.40482555769577276869)
+    assert besseljzero(2,1).ae(5.1356223018406825563)
+    assert besseljzero(1,50).ae(157.86265540193029781)
+    assert besseljzero(10,1).ae(14.475500686554541220)
+    assert besseljzero(0.5,3).ae(9.4247779607693797153)
+    assert besseljzero(2,1,1).ae(3.0542369282271403228)
+    assert besselyzero(0,1).ae(0.89357696627916752158)
+    assert besselyzero(2,1).ae(3.3842417671495934727)
+    assert besselyzero(1,50).ae(156.29183520147840108)
+    assert besselyzero(10,1).ae(12.128927704415439387)
+    assert besselyzero(0.5,3).ae(7.8539816339744830962)
+    assert besselyzero(2,1,1).ae(5.0025829314460639452)
 
 def test_hankel():
     mp.dps = 15
@@ -499,10 +519,10 @@ def test_hyper_2f1():
 def test_hyper_2f1_hard():
     mp.dps = 15
     # Singular cases
-    assert hyp2f1(2,-1,-1,3).ae(0.25)
-    assert hyp2f1(2,-2,-2,3).ae(0.25)
-    assert hyp2f1(2,-1,-1,3,eliminate=False) == 7
-    assert hyp2f1(2,-2,-2,3,eliminate=False) == 34
+    assert hyp2f1(2,-1,-1,3).ae(7)
+    assert hyp2f1(2,-1,-1,3,eliminate_all=True).ae(0.25)
+    assert hyp2f1(2,-2,-2,3).ae(34)
+    assert hyp2f1(2,-2,-2,3,eliminate_all=True).ae(0.25)
     assert hyp2f1(2,-2,-3,3) == 14
     assert hyp2f1(2,-3,-2,3) == inf
     assert hyp2f1(2,-1.5,-1.5,3) == 0.25
@@ -605,8 +625,8 @@ def test_hyper_2f0():
     # Important: cancellation check
     assert hyp2f0((1,6),(5,6),-0.02371708245126284498).ae(0.996785723120804309)
     # Should be exact; polynomial case
-    assert hyp2f0(-2,1,0.5+0.5j) == 0
-    assert hyp2f0(1,-2,0.5+0.5j) == 0
+    assert hyp2f0(-2,1,0.5+0.5j,zeroprec=200) == 0
+    assert hyp2f0(1,-2,0.5+0.5j,zeroprec=200) == 0
     # There used to be a bug in thresholds that made one of the following hang
     for d in [15, 50, 80]:
         mp.dps = d
@@ -854,6 +874,11 @@ def test_gammainc():
     assert gammainc(3,2,2) == 0
     assert gammainc(2,3+j,3-j).ae(-0.28135485191849314194j)
     assert gammainc(4+0j,1).ae(5.8860710587430771455)
+    # GH issue #301
+    assert gammainc(-1,-1).ae(-0.8231640121031084799 + 3.1415926535897932385j)
+    assert gammainc(-2,-1).ae(1.7707229202810768576 - 1.5707963267948966192j)
+    assert gammainc(-3,-1).ae(-1.4963349162467073643 + 0.5235987755982988731j)
+    assert gammainc(-4,-1).ae(1.05365418617643814992 - 0.13089969389957471827j)
     # Regularized upper gamma
     assert isnan(gammainc(0, 0, regularized=True))
     assert gammainc(-1, 0, regularized=True) == inf
@@ -893,12 +918,18 @@ def test_gammainc():
     assert gammainc(5, 10000, 12000).ae('1.1359381951461801687e-4327', abs_eps=0, rel_eps=8*eps)
     # Should use lower gammas
     assert gammainc(10000, 2, 3).ae('8.1244514125995785934e4765')
+    # GH issue 306
+    assert gammainc(3,-1-1j) == 0
+    assert gammainc(3,-1+1j) == 0
+    assert gammainc(2,-1) == 0
+    assert gammainc(2,-1+0j) == 0
+    assert gammainc(2+0j,-1) == 0
 
 def test_gammainc_expint_n():
     # These tests are intended to check all cases of the low-level code
     # for upper gamma and expint with small integer index.
     # Need to cover positive/negative arguments; small/large/huge arguments
-    # for both positive and negative indices, as well as indices 0 and 1 
+    # for both positive and negative indices, as well as indices 0 and 1
     # which may be special-cased
     mp.dps = 15
     assert expint(-3,3.5).ae(0.021456366563296693987)
@@ -964,9 +995,9 @@ def test_gammainc_expint_n():
     assert gammainc(2,3.5).ae(0.13588822540043325333)
     assert gammainc(3,3.5).ae(0.64169439772426814072)
     # Small case; with branch cut
-    assert gammainc(-3,-3.5).ae(0.03595832954467563286 - 0.52359877559829887308j)
+    assert gammainc(-3,-3.5).ae(0.03595832954467563286 + 0.52359877559829887308j)
     assert gammainc(-2,-3.5).ae(-0.88024704597962022221 - 1.5707963267948966192j)
-    assert gammainc(-1,-3.5).ae(4.4637962926688170771 - 3.1415926535897932385j)
+    assert gammainc(-1,-3.5).ae(4.4637962926688170771 + 3.1415926535897932385j)
     assert gammainc(0,-3.5).ae(-13.925353995152335292 - 3.1415926535897932385j)
     assert gammainc(1,-3.5).ae(33.115451958692313751)
     assert gammainc(2,-3.5).ae(-82.788629896730784377)
@@ -982,13 +1013,13 @@ def test_gammainc_expint_n():
     # Asymptotic case; branch cut
     u = gammainc(-3,-350)
     assert u.ae(6.7889565783842895085e+141)
-    assert u.imag.ae(-0.52359877559829887308)
+    assert u.imag.ae(0.52359877559829887308)
     u = gammainc(-2,-350)
     assert u.ae(-2.3692668977889832121e+144)
     assert u.imag.ae(-1.5707963267948966192)
     u = gammainc(-1,-350)
     assert u.ae(8.2685354361441858669e+146)
-    assert u.imag.ae(-3.1415926535897932385)
+    assert u.imag.ae(3.1415926535897932385)
     u = gammainc(0,-350)
     assert u.ae(-2.8856710698020863568e+149)
     assert u.imag.ae(-3.1415926535897932385)
@@ -1011,13 +1042,13 @@ def test_gammainc_expint_n():
     assert gammainc(3,350000000000000000000000).ae('9.2579595072810533625e-152003068666138139677849', abs_eps=0, rel_eps=8*eps)
     u = gammainc(-3,-350000000000000000000000)
     assert u.ae('8.8175642804468234866e+152003068666138139677800')
-    assert u.imag.ae(-0.52359877559829887308)
+    assert u.imag.ae(0.52359877559829887308)
     u = gammainc(-2,-350000000000000000000000)
     assert u.ae('-3.0861474981563882203e+152003068666138139677824')
     assert u.imag.ae(-1.5707963267948966192)
     u = gammainc(-1,-350000000000000000000000)
     assert u.ae('1.0801516243547358771e+152003068666138139677848')
-    assert u.imag.ae(-3.1415926535897932385)
+    assert u.imag.ae(3.1415926535897932385)
     u = gammainc(0,-350000000000000000000000)
     assert u.ae('-3.7805306852415755699e+152003068666138139677871')
     assert u.imag.ae(-3.1415926535897932385)
@@ -1166,6 +1197,142 @@ def test_lambertw():
     assert lambertw(-0.5-0.002j).ae(-0.78917138132659918344 - 0.76743539379990327749j)
     assert lambertw(-0.448+0.4j).ae(-0.11855133765652382241 + 0.66570534313583423116j)
     assert lambertw(-0.448-0.4j).ae(-0.11855133765652382241 - 0.66570534313583423116j)
+    assert lambertw(-0.65475+0.0001j).ae(-0.61053421111385310898+1.0396534993944097723803j)
+    # Huge branch index
+    w = lambertw(1,10**20)
+    assert w.real.ae(-47.889578926290259164)
+    assert w.imag.ae(6.2831853071795864769e+20)
+
+def test_lambertw_hard():
+    def check(x,y):
+        y = convert(y)
+        type_ok = True
+        if isinstance(y, mpf):
+            type_ok = isinstance(x, mpf)
+        real_ok = abs(x.real-y.real) <= abs(y.real)*8*eps
+        imag_ok = abs(x.imag-y.imag) <= abs(y.imag)*8*eps
+        #print x, y, abs(x.real-y.real), abs(x.imag-y.imag)
+        return real_ok and imag_ok
+    # Evaluation near 0
+    mp.dps = 15
+    assert check(lambertw(1e-10), 9.999999999000000000e-11)
+    assert check(lambertw(-1e-10), -1.000000000100000000e-10)
+    assert check(lambertw(1e-10j), 9.999999999999999999733e-21 + 9.99999999999999999985e-11j)
+    assert check(lambertw(-1e-10j), 9.999999999999999999733e-21 - 9.99999999999999999985e-11j)
+    assert check(lambertw(1e-10,1), -26.303186778379041559 + 3.265093911703828397j)
+    assert check(lambertw(-1e-10,1), -26.326236166739163892 + 6.526183280686333315j)
+    assert check(lambertw(1e-10j,1), -26.312931726911421551 + 4.896366881798013421j)
+    assert check(lambertw(-1e-10j,1), -26.297238779529035066 + 1.632807161345576513j)
+    assert check(lambertw(1e-10,-1), -26.303186778379041559 - 3.265093911703828397j)
+    assert check(lambertw(-1e-10,-1), -26.295238819246925694)
+    assert check(lambertw(1e-10j,-1), -26.297238779529035028 - 1.6328071613455765135j)
+    assert check(lambertw(-1e-10j,-1), -26.312931726911421551 - 4.896366881798013421j)
+    # Test evaluation very close to the branch point -1/e
+    # on the -1, 0, and 1 branches
+    add = lambda x, y: fadd(x,y,exact=True)
+    sub = lambda x, y: fsub(x,y,exact=True)
+    addj = lambda x, y: fadd(x,fmul(y,1j,exact=True),exact=True)
+    subj = lambda x, y: fadd(x,fmul(y,-1j,exact=True),exact=True)
+    mp.dps = 1500
+    a = -1/e + 10*eps
+    d3 = mpf('1e-3')
+    d10 = mpf('1e-10')
+    d20 = mpf('1e-20')
+    d40 = mpf('1e-40')
+    d80 = mpf('1e-80')
+    d300 = mpf('1e-300')
+    d1000 = mpf('1e-1000')
+    mp.dps = 15
+    # ---- Branch 0 ----
+    # -1/e + eps
+    assert check(lambertw(add(a,d3)), -0.92802015005456704876)
+    assert check(lambertw(add(a,d10)), -0.99997668374140088071)
+    assert check(lambertw(add(a,d20)), -0.99999999976683560186)
+    assert lambertw(add(a,d40)) == -1
+    assert lambertw(add(a,d80)) == -1
+    assert lambertw(add(a,d300)) == -1
+    assert lambertw(add(a,d1000)) == -1
+    # -1/e - eps
+    assert check(lambertw(sub(a,d3)), -0.99819016149860989001+0.07367191188934638577j)
+    assert check(lambertw(sub(a,d10)), -0.9999999998187812114595992+0.0000233164398140346109194j)
+    assert check(lambertw(sub(a,d20)), -0.99999999999999999998187+2.331643981597124203344e-10j)
+    assert check(lambertw(sub(a,d40)), -1.0+2.33164398159712420336e-20j)
+    assert check(lambertw(sub(a,d80)), -1.0+2.33164398159712420336e-40j)
+    assert check(lambertw(sub(a,d300)), -1.0+2.33164398159712420336e-150j)
+    assert check(lambertw(sub(a,d1000)), mpc(-1,'2.33164398159712420336e-500'))
+    # -1/e + eps*j
+    assert check(lambertw(addj(a,d3)), -0.94790387486938526634+0.05036819639190132490j)
+    assert check(lambertw(addj(a,d10)), -0.9999835127872943680999899+0.0000164870314895821225256j)
+    assert check(lambertw(addj(a,d20)), -0.999999999835127872929987+1.64872127051890935830e-10j)
+    assert check(lambertw(addj(a,d40)), -0.9999999999999999999835+1.6487212707001281468305e-20j)
+    assert check(lambertw(addj(a,d80)), -1.0 + 1.64872127070012814684865e-40j)
+    assert check(lambertw(addj(a,d300)), -1.0 + 1.64872127070012814684865e-150j)
+    assert check(lambertw(addj(a,d1000)), mpc(-1.0,'1.64872127070012814684865e-500'))
+    # -1/e - eps*j
+    assert check(lambertw(subj(a,d3)), -0.94790387486938526634-0.05036819639190132490j)
+    assert check(lambertw(subj(a,d10)), -0.9999835127872943680999899-0.0000164870314895821225256j)
+    assert check(lambertw(subj(a,d20)), -0.999999999835127872929987-1.64872127051890935830e-10j)
+    assert check(lambertw(subj(a,d40)), -0.9999999999999999999835-1.6487212707001281468305e-20j)
+    assert check(lambertw(subj(a,d80)), -1.0 - 1.64872127070012814684865e-40j)
+    assert check(lambertw(subj(a,d300)), -1.0 - 1.64872127070012814684865e-150j)
+    assert check(lambertw(subj(a,d1000)), mpc(-1.0,'-1.64872127070012814684865e-500'))
+    # ---- Branch 1 ----
+    assert check(lambertw(addj(a,d3),1), -3.088501303219933378005990 + 7.458676867597474813950098j)
+    assert check(lambertw(addj(a,d80),1), -3.088843015613043855957087 + 7.461489285654254556906117j)
+    assert check(lambertw(addj(a,d300),1), -3.088843015613043855957087 + 7.461489285654254556906117j)
+    assert check(lambertw(addj(a,d1000),1), -3.088843015613043855957087 + 7.461489285654254556906117j)
+    assert check(lambertw(subj(a,d3),1), -1.0520914180450129534365906 + 0.0539925638125450525673175j)
+    assert check(lambertw(subj(a,d10),1), -1.0000164872127056318529390 + 0.000016487393927159250398333077j)
+    assert check(lambertw(subj(a,d20),1), -1.0000000001648721270700128 + 1.64872127088134693542628e-10j)
+    assert check(lambertw(subj(a,d40),1), -1.000000000000000000016487 + 1.64872127070012814686677e-20j)
+    assert check(lambertw(subj(a,d80),1), -1.0 + 1.64872127070012814684865e-40j)
+    assert check(lambertw(subj(a,d300),1), -1.0 + 1.64872127070012814684865e-150j)
+    assert check(lambertw(subj(a,d1000),1), mpc(-1.0, '1.64872127070012814684865e-500'))
+    # ---- Branch -1 ----
+    # -1/e + eps
+    assert check(lambertw(add(a,d3),-1), -1.075608941186624989414945)
+    assert check(lambertw(add(a,d10),-1), -1.000023316621036696460620)
+    assert check(lambertw(add(a,d20),-1), -1.000000000233164398177834)
+    assert lambertw(add(a,d40),-1) == -1
+    assert lambertw(add(a,d80),-1) == -1
+    assert lambertw(add(a,d300),-1) == -1
+    assert lambertw(add(a,d1000),-1) == -1
+    # -1/e - eps
+    assert check(lambertw(sub(a,d3),-1), -0.99819016149860989001-0.07367191188934638577j)
+    assert check(lambertw(sub(a,d10),-1), -0.9999999998187812114595992-0.0000233164398140346109194j)
+    assert check(lambertw(sub(a,d20),-1), -0.99999999999999999998187-2.331643981597124203344e-10j)
+    assert check(lambertw(sub(a,d40),-1), -1.0-2.33164398159712420336e-20j)
+    assert check(lambertw(sub(a,d80),-1), -1.0-2.33164398159712420336e-40j)
+    assert check(lambertw(sub(a,d300),-1), -1.0-2.33164398159712420336e-150j)
+    assert check(lambertw(sub(a,d1000),-1), mpc(-1,'-2.33164398159712420336e-500'))
+    # -1/e + eps*j
+    assert check(lambertw(addj(a,d3),-1), -1.0520914180450129534365906 - 0.0539925638125450525673175j)
+    assert check(lambertw(addj(a,d10),-1), -1.0000164872127056318529390 - 0.0000164873939271592503983j)
+    assert check(lambertw(addj(a,d20),-1), -1.0000000001648721270700 - 1.64872127088134693542628e-10j)
+    assert check(lambertw(addj(a,d40),-1), -1.00000000000000000001648 - 1.6487212707001281468667726e-20j)
+    assert check(lambertw(addj(a,d80),-1), -1.0 - 1.64872127070012814684865e-40j)
+    assert check(lambertw(addj(a,d300),-1), -1.0 - 1.64872127070012814684865e-150j)
+    assert check(lambertw(addj(a,d1000),-1), mpc(-1.0,'-1.64872127070012814684865e-500'))
+    # -1/e - eps*j
+    assert check(lambertw(subj(a,d3),-1), -3.088501303219933378005990-7.458676867597474813950098j)
+    assert check(lambertw(subj(a,d10),-1), -3.088843015579260686911033-7.461489285372968780020716j)
+    assert check(lambertw(subj(a,d20),-1), -3.088843015613043855953708-7.461489285654254556877988j)
+    assert check(lambertw(subj(a,d40),-1), -3.088843015613043855957087-7.461489285654254556906117j)
+    assert check(lambertw(subj(a,d80),-1), -3.088843015613043855957087 - 7.461489285654254556906117j)
+    assert check(lambertw(subj(a,d300),-1), -3.088843015613043855957087 - 7.461489285654254556906117j)
+    assert check(lambertw(subj(a,d1000),-1), -3.088843015613043855957087 - 7.461489285654254556906117j)
+    # One more case, testing higher precision
+    mp.dps = 500
+    x = -1/e + mpf('1e-13')
+    ans = "-0.99999926266961377166355784455394913638782494543377383"\
+    "744978844374498153493943725364881490261187530235150668593869563"\
+    "168276697689459394902153960200361935311512317183678882"
+    mp.dps = 15
+    assert lambertw(x).ae(ans)
+    mp.dps = 50
+    assert lambertw(x).ae(ans)
+    mp.dps = 150
+    assert lambertw(x).ae(ans)
 
 def test_meijerg():
     mp.dps = 15
@@ -1280,3 +1447,14 @@ def test_spherharm():
 def test_qfunctions():
     mp.dps = 15
     assert qp(2,3,100).ae('2.7291482267247332183e2391')
+
+def test_issue_239():
+    mp.prec = 150
+    x = ldexp(2476979795053773,-52)
+    assert betainc(206, 385, 0, 0.55, 1).ae('0.99999999999999999999996570910644857895771110649954')
+    mp.dps = 15
+    try:
+        u = hyp2f1(-5,5,0.5,0.5)
+        raise AssertionError("hyp2f1(-5,5,0.5,0.5) (failed zero detection)")
+    except (mp.NoConvergence, ValueError):
+        pass
